from .utilities import Utilities, ESICentroLoginException

import logging

logger = logging.getLogger(__name__)

class ESICentroLoginResponse:
    __slots__ = [
        "_email",
        "_user_id",
        "_access_token",
    ]

    def __init__(self, response):

        if response["statu"]:
            self.email = response["user"]["email"]
            self.user_id = response["user"]["id"]
            self.access_token = response["user"]["token"]
        else: 
            #print("***ESICentroLoginResponse Error Handling required***")
            raise ESICentroLoginException(response)

    @property
    def email(self):
        return self._email

    @email.setter
    def email(self, value):
        self._email = value

    @property
    def user_id(self):
        return self._user_id

    @user_id.setter
    def user_id(self, value):
        self._user_id = value

    @property
    def access_token(self):
        return self._access_token

    @access_token.setter
    def access_token(self, value):
        self._access_token = value


class RequestThermostatIDsResponse:
    __slots__ = ["_device_ids"]

    def __init__(self, response):
        self.device_ids = []
        if isinstance(response, dict):
            for (i, key) in enumerate(response):
                if isinstance(response[key], list):
                    for devices in response[key]:
                        self.device_ids.append(devices["device_id"])

    @property
    def device_ids(self):
        return self._device_ids

    @device_ids.setter
    def device_ids(self, value):
        self._device_ids = value


class RequestThermostatDataResponse:
    __slots__ = ["_device_data"]

    def __init__(self, response):
        self.device_data = response["devices"][0]
        Utilities.convert_temperatures(self)

    @property
    def device_data(self):
        return self._device_data

    @device_data.setter
    def device_data(self, value):
        self._device_data = value


class SetThermostatTempResponse:
    __slots__ = ["_status", "_error_code", "_message", "_tcpCode"]

    def __init__(self, response):
        if isinstance(response, dict):

            self.status = response["statu"]
            self.error_code = response["error_code"]
            self.message = response["message"]
            self.tcpCode = response["tcpCode"]

    @property
    def status(self):
        return self._status

    @status.setter
    def status(self, value):
        self._status = value

    @property
    def message(self):
        return self._message

    @message.setter
    def message(self, value):
        self._message = value

    @property
    def error_code(self):
        return self._error_code

    @error_code.setter
    def error_code(self, value):
        self._error_code = value

    @property
    def tcpCode(self):
        return self._tcpCode

    @tcpCode.setter
    def tcpCode(self, value):
        self._tcpCode = value



import logging
import random
import aiohttp
import asyncio

import logging

logger = logging.getLogger(__name__)

class Utilities:
    __slots__ = []

    @classmethod
    async def dictionary_to_object(
        cls,
        data,
        # property_type,
        # response_properties,
        # parent_classes=[],
        # indent=0,
        # is_top_level=False,
    ):

        if isinstance(data, dict):  # returned data is a dictionary
            for (i, key) in enumerate(data):
                print("i: ", i)
                print("key: ", key)
                if isinstance(data[key], dict):  # returned data is a dictionary
                    print("data[key]: ", data[key])
                    parent_classes = [key]
                    parent_classes.append(data[key])
                    print("parent_classes: ", parent_classes)

    @classmethod
    async def make_http_request(
        cls,
        session,
        method,
        url,
        headers=None,
        params=None,
        json_=None,
        timeout=5,
    ):

        async with session.request(
            method,
            url,
            headers=headers,
            params=params,
            json=json_,
            timeout=timeout,
        ) as response:
            try:
                response_json = await response.json(content_type="text/json")
                response.raise_for_status()

                return response_json
            except aiohttp.ClientResponseError as cre:
                raise ESICentroError(cre) from cre

    @classmethod
    async def process_http_response(cls, response, response_class):
        return response_class(response)

    async def randomMsgId() -> str:
        """randomMsgId

        Returns:
            str: a random 4 character long string, required by some requests
        """
        return "".join(random.choice("0123456789abcdef") for n in range(4))

    def convert_to_esi_temp(temp):
        return int(float(temp) * 10)

    def convert_from_esi_temp(temp):
        return float(temp) / 10

    def convert_temperatures(self):
        updates = {}
        updates.update(
            Utilities.convert_preset_temps(self.device_data["presetsTemp"])
        )
        updates.update(
            {
                "current_temprature": Utilities.convert_from_esi_temp(
                    self.device_data["current_temprature"]
                )
            }
        )
        updates.update(
            {
                "inside_temparature": Utilities.convert_from_esi_temp(
                    self.device_data["inside_temparature"]
                )
            }
        )

        self.device_data.update(updates)

    def convert_preset_temps(preset_temps):
        temps = preset_temps.split("|")

        updates = {}
        updates.update({"presetHomeTemp": Utilities.convert_from_esi_temp(temps[0])})
        updates.update(
            {"presetSleepTemp": Utilities.convert_from_esi_temp(temps[1])}
        )
        updates.update({"presetAwayTemp": Utilities.convert_from_esi_temp(temps[2])})

        return updates

class ESICentroError(Exception):
    pass

class ESICentroException(ESICentroError):
    """CentroError

    Args:
        Exception ([type]): [description]
    """

    def __init__(self, message):
        self.message = message
        super(ESICentroException, self).__init__(self.message)

    @property
    def message(self):
        return self._message

    @message.setter
    def message(self, value):
        self._message = value

class ESICentroLoginException(ESICentroError):
    """CentroError

    Args:
        Exception ([type]): [description]
    """

    def __init__(self, message):
        self.message = message
        self.error_message = message["message"]
        super(ESICentroLoginException, self).__init__(self.error_message)

    @property
    def message(self):
        return self._message

    @message.setter
    def message(self, value):
        self._message = value



from typing import Any, Dict, List, Optional

import logging

logger = logging.getLogger(__name__)

class ESICentroDevice:
    """[summary]"""

    __slots__ = [
        #"device_id",
        "device_type",
        "device_preset_temps",
        "online",
        #"device_name",
        "current_temperature",
        "inside_temperature",
        "program_mode",
        "dis_on_off",
        "work_mode",
        "th_work",
        "fornt_work_mode_tempara",
        "fwd",
        "event_count",
        "device_mac",
        "device_version",
        "new_version",
        "gateway_standby",
        "lock_screen",
        "password",
        "boost_duration",
        "boost_start",
        "remaining",
        "departure_time",
        "return_time",
        "device_ip_address",
        "device_sak",
        "device_sn",
        "super_device_type",
        "upgrading",
        "encryption",
        "holiday_mode",
        "holiday_record",
        "holiday_temparature",
        "command_type1",
        "command_type2",
        "auto_temp",
        "gateway_type",
        "app_online_count",
        "device_time",
        "upgrade",
        "app_request_binding_time",
    ]

    def __init__(self, device_data):

        device_data = device_data[0]
        self.device_id = device_data["device_id"]
        self.device_type = device_data["device_type"]
        self.device_preset_temps = device_data["presetsTemp"]
        self.online = device_data["online"]
        self.device_name = device_data["device_name"]
        self.current_temperature = device_data["current_temprature"]
        self.inside_temperature = device_data["inside_temparature"]
        self.program_mode = device_data["program_mode"]
        self.dis_on_off = device_data["disOnOff"]
        self.work_mode = device_data["work_mode"]
        self.th_work = device_data["th_work"]
        self.fornt_work_mode_tempara = device_data["frontWorkModeTempara"]
        self.fwd = device_data["fwD"]
        self.event_count = device_data["event_count"]
        self.device_mac = device_data["device_mac"]
        self.device_version = device_data["device_version"]
        self.new_version = device_data["new_version"]
        self.gateway_standby = device_data["gatewayStandby"]
        self.lock_screen = device_data["lockScreen"]
        self.password = device_data["password"]
        self.boost_duration = device_data["boost_duration"]
        self.boost_start = device_data["boost_start"]
        self.remaining = device_data["remaining"]
        self.departure_time = device_data["departure_time"]
        self.return_time = device_data["return_time"]
        self.device_ip_address = device_data["device_ipAddress"]
        self.device_sak = device_data["device_sak"]
        self.device_sn = device_data["device_sn"]
        self.super_device_type = device_data["superDeviceType"]
        self.upgrading = device_data["upgrading"]
        self.encryption = device_data["encryption"]
        self.holiday_mode = device_data["holidayMode"]
        self.holiday_record = device_data["holidayRecord"]
        self.holiday_temparature = device_data["holidayTemparature"]
        self.command_type1 = device_data["command_type1"]
        self.command_type2 = device_data["command_type2"]
        self.auto_temp = device_data["autoTemp"]
        self.gateway_type = device_data["gatewayType"]
        self.app_online_count = device_data["appOnlineCount"]
        self.device_time = device_data["deviceTime"]
        self.upgrade = device_data["upgrade"]
        self.app_request_binding_time = device_data["appRequestBindingTime"]

    @property
    def device_id(self):
        return self.device_id

    @property
    def device_name(self):
        return self.device_name



HTTP_GET = "GET"
HTTP_POST = "POST"

CENTRO_EMAIL = "email"
CENTRO_PASSWORD = "password"

CENTRO_ENDPOINT_THERMOSTAT = "thermostat"
CENTRO_DEVICE_TYPES = "02"
CENTRO_DEVICE_TYPE = "device_type"
CENTRO_DEVICE = "device_id"
CENTRO_USER = "user_id"
CENTRO_ACCESS_TOKEN = "token"

CENTRO_BASE_URL = "https://esiheating.uksouth.cloudapp.azure.com/centro/"
CENTRO_LOGIN_URL = "login"
CENTRO_DEVICELIST_URL = "getDeviceList"
CENTRO_SETWORKMODE_URL = "setThermostatWorkModeNew"
CENTRO_DEVICESETTINGS_URL = "getDeviceSettingInfo"
CENTRO_SETTEMP_URL = "setThermostatTemprature"
CENTRO_MODIFYDEVICENAME_URL = "modifyDeviceName"
CENTRO_PRESETEMPS_URL = "moDevicePreTemp"
CENTRO_SETPROGBYDAY_URL = "setProgrammeByDay"
CENTRO_SETPROGITEMCOUNT_URL = "setProgrammeItemCountMode"
CENTRO_SETBOOSTMODE_URL = "setThermostatBoostModeNew"

CENTRO_WORKMODE_SCHEDULE = "0"
CENTRO_WORKMODE_5_2_SCHEDULE = "1"
CENTRO_WORKMODE_24HR_SCHEDULE = "2"
CENTRO_WORKMODE_OFF = "4"
CENTRO_WORKMODE_MANUAL = "5"

CENTRO_ACCESS_TOKEN_TIMEOUT = "1800"


from os import access
from typing import Optional
from aiohttp import ClientSession, ClientResponseError

from .const import (
    CENTRO_BASE_URL,
    CENTRO_DEVICE_TYPE,
    CENTRO_DEVICE_TYPES,
    CENTRO_EMAIL,
    CENTRO_LOGIN_URL,
    CENTRO_PASSWORD,
    CENTRO_SETWORKMODE_URL,
    HTTP_GET,
    HTTP_POST,
    CENTRO_DEVICELIST_URL,
    CENTRO_DEVICESETTINGS_URL,
    CENTRO_USER,
    CENTRO_ACCESS_TOKEN,
    CENTRO_DEVICE
)

from .device import ESICentroDevice
from .responses import ESICentroLoginResponse, SetThermostatTempResponse
from .responses import RequestThermostatIDsResponse
from .responses import RequestThermostatDataResponse
from .utilities import ESICentroException, Utilities

import logging
logger = logging.getLogger(__name__)

class ESICentroClient:
    """[summary]"""

    __slots__ = [
        "_email",
        "_password",
        "_user_id",
        "_access_token",
        "_session_details",
        "_device_ids",
    ]

    def __init__(
        self, email, password, session_details, user_id=None, access_token=None
    ):

        if session_details is not None:
            self.session_details = session_details
        else:
            self.session_details = ClientSession()

        self.user_id = user_id
        self.password = password
        self.email = email
        self.access_token = access_token

    @property
    def user_id(self):
        return self._user_id

    @user_id.setter
    def user_id(self, value):
        self._user_id = value

    @property
    def password(self):
        return self._password

    @password.setter
    def password(self, value):
        self._password = value

    @property
    def email(self):
        return self._email

    @email.setter
    def email(self, value):
        self._email = value

    @property
    def access_token(self):
        return self._access_token

    @access_token.setter
    def access_token(self, value):
        self._access_token = value

    @property
    def session_details(self):
        return self._session_details

    @session_details.setter
    def session_details(self, value):
        self._session_details = value

    @property
    def device_ids(self):
        return self._device_ids

    @device_ids.setter
    def device_ids(self, value):
        self._device_ids = value

    async def login(self, timeout=5):
        response = await Utilities.make_http_request(
            self.session_details,
            HTTP_POST,
            CENTRO_BASE_URL + CENTRO_LOGIN_URL,
            params={
                CENTRO_EMAIL: self.email,
                CENTRO_PASSWORD: self.password,
            },
            timeout=timeout,
        )
        try:
            authorize_response = await Utilities.process_http_response(
                response, ESICentroLoginResponse
            )
        except ESICentroException as ex:
            logger.exception(ex)

        return authorize_response

    async def request_thermostat_ids(self, timeout=5):
        response = await Utilities.make_http_request(
            self.session_details,
            HTTP_GET,
            CENTRO_BASE_URL + CENTRO_DEVICELIST_URL,
            params={
                CENTRO_DEVICE_TYPE: CENTRO_DEVICE_TYPES,
                CENTRO_USER: self.user_id,
                CENTRO_ACCESS_TOKEN: self.access_token,
            },
            timeout=timeout,
        )

        processed_response = await Utilities.process_http_response(
            response, RequestThermostatIDsResponse
        )

        self.device_ids = processed_response.device_ids

        return processed_response

    async def request_thermostat_data(self, device_id, timeout=5):
        response = await Utilities.make_http_request(
            self._session_details,
            HTTP_GET,
            CENTRO_BASE_URL + CENTRO_DEVICESETTINGS_URL,
            params={
                CENTRO_DEVICE: device_id,
                CENTRO_USER: self.user_id,
                CENTRO_ACCESS_TOKEN: self.access_token,
            },
            timeout=timeout,
        )

        processed_response = await Utilities.process_http_response(
            response, RequestThermostatDataResponse
        )

        return processed_response

    async def set_thermostat_temperature(
        self, device_id, current_temprature, work_mode, timeout=5
    ):
        messageId = await Utilities.randomMsgId()
        current_temperature = Utilities.convert_to_esi_temp(current_temprature)
        response = await Utilities.make_http_request(
            self.session_details,
            HTTP_POST,
            CENTRO_BASE_URL + CENTRO_SETWORKMODE_URL,
            params={
                CENTRO_DEVICE: device_id,
                "current_temprature": current_temperature,
                CENTRO_USER: self.user_id,
                CENTRO_ACCESS_TOKEN: self.access_token,
                "work_mode": work_mode,
                "messageId": messageId,
            },
            timeout=timeout,
        )

        processed_response = await Utilities.process_http_response(
            response, SetThermostatTempResponse
        )

        return processed_response



# setup.py
from setuptools import setup, find_packages

VERSION = '0.0.2'
DESCRIPTION = 'Python 3 module to use with ESI Centro Thermostats'
LONG_DESCRIPTION = "A python module for accessing ESI Centro Thermostats.  This package was primarily created for use with HomeAssistant."

# Setting up
setup(
    name="pyesicentro",
    version=VERSION,
    author="John Matin McLaughlin",
    author_email="johnmartinmclaughlin@gmail.com",
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    packages=["pyesicentro"],
    install_requires=[],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Programming Language :: Python :: 3",
    ]
)